# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""
   Tools for developpers :
   - search messages
"""

__revision__ = "$Id: as_devtools.py 3255 2008-04-10 17:13:17Z courtois $"


# ----- differ messages translation
def _(mesg):
   return mesg

import sys
import os
import re
import pprint
import cPickle
from glob import glob
from sets import Set

from as_config    import ASTER_CONFIG
from progress     import Progress


#------------------------------------------------------------------------------
# -----------------------------------------------------------------------------
class MessageError(Exception):
   pass

class MESSAGE_MANAGER(object):
   """Classe pour rcuprer des informations sur le catalogue de messages.
   """
   def __init__(self, repref, fort, pyt, cache_dict,
                      verbose=True, force=False, ignore_comments=True, debug=False):
      """Initialisations
      """
      if type(fort) not in (list, tuple):
         fort = [fort,]
      self.repref          = repref
      self.fort            = [os.path.join(repref, rep) for rep in fort]
      self.pyt             = os.path.join(repref, pyt)
      self.verbose         = verbose
      self.debug           = debug
      self.force           = force
      self.message_dir     = 'Messages'
      self.cache_dict      = cache_dict
      self.ignore_comments = ignore_comments
      self.wrk_fort        = os.path.join(os.getcwd(), 'F')
      self.wrk_pyt         = os.path.join(os.getcwd(), 'PY')
      if self.verbose:
         print 'Repertoires :\n   REF=%s\n   FORTRAN=%s\n   PYTHON=%s' % (self.repref, self.fort, self.pyt)

      self.l_cata            = self._get_list_cata()
      self.l_src, self.l_pyt = self._get_list_src()
      self._build_regexp()
      self._build_dict()
      self.print_stats()

# -----------------------------------------------------------------------------
   def _filename_cata(self, cata):
      """Retourne le nom du fichier associ  un catalogue."""
#       name = os.path.join(self.wrk_pyt, cata + '.py')
#       if os.path.isfile(name):
#          print 'Using modified catalogue : %s' % name
#       else:
      name = os.path.join(self.pyt, self.message_dir, cata + '.py')
      return name

# -----------------------------------------------------------------------------
   def _id2filename(self, msgid):
      """Retourne modelisa5  partir de MODELISA5_46"""
      mat = re.search('([A-Z]+[0-9]*)_([0-9]+)', msgid)
      if mat == None:
         raise MessageError, 'invalid message id : %s' % msgid
      cata, num = mat.groups()
      return self._filename_cata(cata.lower()), int(num)

# -----------------------------------------------------------------------------
   def _filename2id(self, fcata, num):
      """Retourne MODELISA5_46  partir de modelisa5"""
      return '%s_%d' % (os.path.splitext(os.path.basename(fcata))[0].upper(), num)

# -----------------------------------------------------------------------------
   def makedirs(self):
      """Cre les rpertoires pour les fichiers modifis
      """
      for d in (self.wrk_fort, self.wrk_pyt):
         if not os.path.isdir(d):
            os.makedirs(d)

# -----------------------------------------------------------------------------
   def _get_list_cata(self):
      """Retourne la liste des catalogues de messages.
      """
      l_mod = Set([os.path.splitext(os.path.basename(f))[0]
                        for f in glob(self._filename_cata('*'))])
      l_mod.difference_update(['__init__', 'context_info'])
      l_cata = list(l_mod)
      l_cata.sort()
      return l_cata

# -----------------------------------------------------------------------------
   def check_cata(self):
      """Vrifie les catalogues.
      """
      print '%5d catalogues dans %s' % (len(self.l_cata), self.pyt)
      ntot = 0
      all_msg = []
      for f in self.l_cata:
         try:
            cata = CATA(self._filename_cata(f))
            ntot += cata.get_nb_msg()
            all_msg.extend([self._filename2id(f, i) for i in cata])
            print cata
         except MessageError, msg:
            print msg
            self.l_cata.remove(f)
      all_msg = Set(all_msg)
      print '%6d messages dans les catalogues' % ntot
            
      # messages jamais appels
      used = Set(self.d_msg_call.keys())
      unused =  list(all_msg.difference(used))
      unused.sort()
      print '%6d messages inutiliss' % len(unused)
      if self.debug:
         print unused
      
      union = used.union(all_msg)
      inexist = list(used.difference(all_msg))
      inexist.sort()
      print '%6d messages appels absents des catalogues' % len(inexist)
      if self.debug:
         print inexist
      print '%6d messages total (union pour verif)' % len(union)

      return unused, inexist


# -----------------------------------------------------------------------------
   def _build_regexp(self):
      """Construit la liste des expressions rgulires pour la recherche des messages.
      """
      l_regexp = []
      for cata in self.l_cata:
         l_regexp.append('%s_[0-9]+' % cata.upper())
      self.regexp = re.compile('|'.join(l_regexp))

# -----------------------------------------------------------------------------
   def _get_list_src(self):
      """Retourne la liste des routines fortran et python.
      """
      l_f = []
      for dsrc in self.fort:
         l_f.extend(glob(os.path.join(dsrc, '*.f')))
         l_f.extend(glob(os.path.join(dsrc, '*', '*.f')))
      l_pyt = glob(os.path.join(self.pyt, '*', '*.py'))
      if self.verbose:
         print '%5d routines fortran dans %s' % (len(l_f), self.fort)
         print '%5d modules python   dans %s' % (len(l_pyt), self.pyt)
      return l_f, l_pyt

# -----------------------------------------------------------------------------
   def search_message(self, fich):
      """Retourne les messages utiliss dans 'fich'.
      """
      try:
         txt = open(fich, 'r').read()
      except IOError, msg:
         print _('Error with file %s : %s') % (fich, msg)
         return []
      if self.ignore_comments:
         ext = os.path.splitext(fich)[-1]
         assert ext in ('.f', '.py'), 'unknown type : %s' % str(ext)
         if ext == '.f':
            reg_ign = re.compile('(^[A-Z!#].*$)')
         elif ext == '.py':
            reg_ign = re.compile('(#.*$)')
         txt = reg_ign.sub('', txt)
      l_msg = self.regexp.findall(txt)
      # verif
      l_res = []
      for msg in l_msg:
         spl = msg.split('_')
         assert len(spl) == 2, 'ERROR invalid : %s' % msg
         msg = '%s_%d' % (spl[0], int(spl[1]))
         l_res.append(msg)
         
      return l_res

# -----------------------------------------------------------------------------
   def _build_dict(self):
      """Construit les dictionnaires :
         fichier source : liste des messsages appels
         message : liste des fichiers appelant ce message
      """
      # est-ce dans le cache ?
      if not self.force and os.path.isfile(self.cache_dict) \
                        and os.stat(self.cache_dict).st_size > 0:
         if self.verbose:
            print 'Load dicts from cache (%s)...' % self.cache_dict
         pick = open(self.cache_dict, 'r')
         self.d_msg_used = cPickle.load(pick)
         self.d_msg_call = cPickle.load(pick)
         pick.close()

      else:   
         self.d_msg_used = {}
         self.d_msg_call = { 'SUPERVIS_40' : ['astk',] }
         l_all = self.l_src + self.l_pyt
         if self.verbose:
            p = Progress(maxi=len(l_all), format='%5.1f %%',
                        msg='Analyse des sources... ')
         for i, f in enumerate(l_all):
            if self.verbose:
               p.Update(i)
#             key = f.replace(self.repref, '')
            key = re.sub('^%s/*', '', f)
            lm = self.search_message(f)
            if len(lm) > 0:
               self.d_msg_used[key] = lm
               for msg in lm:
                  self.d_msg_call[msg] = self.d_msg_call.get(msg, []) + [key,]
         
         if self.verbose:
            p.End()
         
         pick = open(self.cache_dict, 'w')
         cPickle.dump(self.d_msg_used, pick)
         cPickle.dump(self.d_msg_call, pick)
         pick.close()

# -----------------------------------------------------------------------------
   def print_stats(self):
      """Affiche les stats sur les donnes lues/construites
      """
      if self.verbose:
         print '%6d messages appels dans les sources' % len(self.d_msg_call.keys())
         print '%6d fichiers sources appellent le catalogue de messages' % len(self.d_msg_used.keys())

# -----------------------------------------------------------------------------
   def move(self, oldid, dest, reuse_hole=True):
      """Dplace le message 'oldid' dans le catalogue 'dest'.
      """
      if self.verbose:
         print '--- moving "%s" into "%s"' % (oldid, dest)
      # catalogue objects
      old_f, old_num = self._id2filename(oldid)
      new_f = self._filename_cata(dest.lower())
      
      # have these catalogues been already changed ?
      fmod = os.path.join(self.wrk_pyt, os.path.basename(old_f))
      if os.path.isfile(fmod):
         print 'from %s' % fmod
         old_f = fmod
      fmod = os.path.join(self.wrk_pyt, os.path.basename(new_f))
      if os.path.isfile(fmod):
         print 'from %s' % fmod
         new_f = fmod
         
      old_cata = CATA(old_f)
      new_cata = CATA(new_f)
      if self.verbose:
         print old_cata
         print new_cata
      new_num = new_cata.get_new_id(reuse_hole)
      if new_num < 0:
         raise MessageError,  'no message id available in %s' % new_f
      newid = self._filename2id(new_f, new_num)
      
      # check message existence
      if old_cata[old_num] == None:
         raise MessageError, 'message %s inexistant !' % oldid
      
      new_cata[new_num] = old_cata[old_num]
      del old_cata[old_num]

      # write modified catalogues
      self.makedirs()
      fout = os.path.join(self.wrk_pyt, os.path.basename(old_f))
      content = old_cata.content()
      open(fout, 'w').write(content)
      fout = os.path.join(self.wrk_pyt, os.path.basename(new_f))
      content = new_cata.content()
      open(fout, 'w').write(content)
      print 'Nouveau fichier : %s' % fout
      
      # modify source using 'oldid' message
      l_src = self.d_msg_call.get(oldid, [])
      for f in l_src:
         ext = os.path.splitext(f)[-1]
         rdest = self.wrk_fort
         if ext == '.py':
            rdest = self.wrk_pyt
         # already changed ?
         fmod = os.path.join(rdest, os.path.basename(f))
         if os.path.isfile(fmod):
            print 'from %s' % fmod
            f = fmod
         txt = open(os.path.join(self.repref, f), 'r').read()
         new = re.sub('%s([\'\"]+)' % oldid, newid + r'\1', txt)
         fout = os.path.join(rdest, os.path.basename(f))
         open(fout, 'w').write(new)
         print 'Nouveau fichier : %s' % fout

# -----------------------------------------------------------------------------
   def who_use(self, msg):
      """Qui utilise le message 'msg' ?
      """
      return tuple(self.d_msg_call.get(msg.upper(), []))

# -----------------------------------------------------------------------------
   def get_key(self, sub):
      """Retourne la cl dans d_msg_used correspondant  'sub'.
      Seule la premire est retourne si jamais il y en avait plusieurs.
      """
      l_allsub = self.d_msg_used.keys()
      l_sub = [f for f in l_allsub if f.split(os.sep)[-1] == sub]
      if len(l_sub) > 1:
         print 'Plusieurs routines correspondent : %s' % ', '.join(l_sub)
         print 'On utilise la premire valeur.'
      if len(l_sub) == 0:
         result = None
      else:
         result = l_sub[0]
      return result

# -----------------------------------------------------------------------------
   def which_msg(self, sub):
      """Quels sont les messages utiliss par la routine 'sub' ?
      """
      key = self.get_key(sub)
      return tuple(self.d_msg_used.get(key, []))


# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
template_cata_header = """# -*- coding: iso-8859-1 -*-
#            CONFIGURATION MANAGEMENT OF EDF VERSION
# ======================================================================
# COPYRIGHT (C) 1991 - 2007  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY  
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY  
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR     
# (AT YOUR OPTION) ANY LATER VERSION.                                                  
#                                                                       
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT   
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF            
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU      
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.                              
#                                                                       
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE     
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,         
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.        
# ======================================================================

def _(x) : return x

cata_msg = {"""

template_cata_footer = """
}
"""

template_cata_msg = '''
%(msgid)s : _("""%(text)s"""),'''


# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
class CATA:
   """Classe reprsentant un catalogue de messages.
   Mthodes attendues :
      - nombre de messages,
      - indice du dernier messages,
      - indice libre,
      - ajouter/supprimer un message,
      - ...
   """
   def __init__(self, fcata):
      """Initialisation
      """
      self.fcata = fcata
      self.cata_msg = {}
      try:
         d = {}
         execfile(fcata, d)
         self.cata_msg = d['cata_msg']
      except Exception, msg:
         print '-'*80+'\n', msg, '\n'+'-'*80
         raise MessageError, 'ERROR with : %s' % self.fcata

# -----------------------------------------------------------------------------
   def get_nb_msg(self):
      """Nombre de messages."""
      return len(self.cata_msg)

# -----------------------------------------------------------------------------
   def get_last_id(self):
      """Indice du dernier message."""
      return max(self.cata_msg.keys() or [0,])

# -----------------------------------------------------------------------------
   def get_new_id(self, reuse_hole=True):
      """Indice libre. Si 'end', on prend systmatiquement  la fin,
      sinon on cherche aussi dans les trous.
      """
      if not reuse_hole:
         start = self.get_last_id()
      else:
         start = 1
      all = Set(range(start, 100))
      free = all.difference(self.cata_msg.keys())
      if len(free) == 0:
         new = -1
      else:
         new = min(free)
      return new

#-------------------------------------------------------------------------------
   def __getitem__(self, key):
      """Retourne le contenu du message ou None.
      """
      return self.cata_msg.get(key, None)

#-------------------------------------------------------------------------------
   def __setitem__(self, key, msg):
      """Ajoute le message 'msg'  l'indice 'key'.
      """
      if self[key] != None:
         raise MessageError, 'message %s already exists !' % key
      self.cata_msg[key] = msg

#-------------------------------------------------------------------------------
   def __delitem__(self, key):
      """Supprime le message d'indice 'key'.
      """
      if self[key] == None:
         raise MessageError, 'message %s does not exist !' % key
      del self.cata_msg[key]

# -----------------------------------------------------------------------------
   def __repr__(self):
      """Rsum du catalogue.
      """
      return '%3d messages (last=%3d, next=%3d) dans %s' % \
            (self.get_nb_msg(), self.get_last_id(), self.get_new_id(), self.fcata)

# -----------------------------------------------------------------------------
   def __iter__(self):
      """Itration sur les id des messages.
      """
      return iter(self.cata_msg)

# -----------------------------------------------------------------------------
   def get_cmodif(self):
      """Retourne la carte MODIF.
      """
      cmodif = None
      fobj = open(self.fcata, 'r')
      for line in fobj:
         if re.search('#@ +MODIF|#@ +AJOUT', line):
            cmodif = line.replace(os.linesep, '')
            break
      fobj.close()
      if cmodif == None:
         raise MessageError, 'invalid header "#@ MODIF/AJOUT..." in %s' % self.fcata
      return cmodif

# -----------------------------------------------------------------------------
   def content(self):
      """Contenu du catalogue "reconstruit".
      """
      txt = [self.get_cmodif(),]
      txt.append(template_cata_header)
      l_ind = self.cata_msg.keys()
      l_ind.sort()
      for msgid in l_ind:
         txt.append(template_cata_msg % {'msgid' : msgid, 'text' : self.cata_msg[msgid]})
      txt.append(template_cata_footer)
      return os.linesep.join(txt)

# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
def GetMessageInfo(run, *args):
   """Return info about Code_Aster messages.
   """
   REPREF = os.path.join(run.aster_root, run['aster_vers'])
   if not run.get('config'):
      fconf = os.path.join(REPREF, 'config.txt')
   else:
      fconf = os.path.abspath(run['config'])
   conf = ASTER_CONFIG(fconf, run)

   if not run['local']:
      run.Mess(_('This operation only works on local source files. "local" mode forced'), _('<A>_ALARM'))

   bibfor = [conf['SRCFOR'][0], conf['SRCFERM'][0]]
   if conf['SRCF90'][0] != '':
      bibfor.append(conf['SRCF90'][0])
   else:
      run.Mess(_('Fortran 90 source files are not used on this machine (see %s)') \
            % conf.filename, _('<A>_ALARM'))
   bibpyt = conf['SRCPY'][0]

   args = list(args)
   named_actions = ('check', 'move',)
   action = None
   
   if len(args) > 0 and args[0] in named_actions:
      action = args.pop(0)
   elif len(args) == 0:
      run.Mess(_('You must choose an operation from %s or give a subroutine name or a message ID') \
            % repr(named_actions), _('<F>_INVALID_ARGUMENT'))
   
   msgman = MESSAGE_MANAGER(repref=REPREF, fort=bibfor, pyt=bibpyt, 
                            cache_dict=os.path.join(run['cache_dir'], 'messages_dicts.pick'),
                            force=run['force'], verbose=not run['silent'],
                            debug=run['debug'])
   
   if action == 'check':
      unused, inexist = msgman.check_cata()
      print 'Messages inutiliss :'
      print unused
      print 'Messages inexistants :'
      print inexist

   elif action == 'move':
      if len(args) != 2:
         run.parser.error(
               _("'--%s %s' requires 2 arguments") % (run.current_action, action))
      msgman.move(*args)
   
   else:
      print
      fmt = '%12s : %s'
      for obj in args:
         l_msg = msgman.which_msg(obj)
         if len(l_msg) > 0:
            print fmt % (obj, l_msg)
         l_sub = msgman.who_use(obj)
         if len(l_sub) > 0:
            print fmt % (obj, l_sub)


# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
if __name__ == '__main__':

   # Pour une utilisation hors de as_run :
   #import sys
   #sys.path.append('.../ASTK/ASTK_SERV/lib')
   #from as_devtools import *

   msgman = MESSAGE_MANAGER(repref='/usr/local/aster/STA9.3', fort=('bibfor', 'bibf90'), pyt='bibpyt', 
                            cache_dict=os.path.join('/tmp', 'messages_dicts.pick'),
                            force=False, debug=False)
   unused, inexist = msgman.check_cata()




