# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""
This module gives functions for services which run as_run itself
in a separate process.

The functions are called by an ASTER_RUN object.
"""

__revision__ = "$Id: $"


# ----- differ messages translation
def _(mesg):
   return mesg

import os
from as_serv_normal  import AS_SERV_NORMAL
from as_serv_special import AS_SERV_SPECIAL

#-------------------------------------------------------------------------------
def SetParser(run):
   """Configure the command-line parser, add options name to store to the list,
   set actions informations.
   run : ASTER_RUN object which manages the execution
   """
   acts_descr = {
      'normal' : {
         'method' : Normal,
         'syntax' : 'user@mach:/nom_profil.export',
         'help'   : 'start an execution (as_run in a separate process)'
      },
      'special' : {
         'method' : Special,
         'syntax' : 'user@mach:/nom_profil.export',
         'help'   : 'start a special execution (stanley, convbase, meshtool)'
      },
   }
   run.SetActions(
         actions_descr = acts_descr,
         actions_order = ['normal', 'special'],
         group_options=False,
         options_descr = {},
   )

#-------------------------------------------------------------------------------
def Normal(run, *args):
   """Start an execution, another as_run, in a separate process.
   """
   # ----- check argument
   if len(args) != 1:
      run.parser.error(_("'--%s' requires one argument") % run.current_action)

   service = AS_SERV_NORMAL(run, args[0])
   iret, output = service.start()
   run.Sortie(iret)

#-------------------------------------------------------------------------------
def Special(run, *args):
   """Start a special service : open results database, mesh converter,
   database converter.
   """
   # ----- check argument
   if len(args) != 1:
      run.parser.error(_("'--%s' requires one argument") % run.current_action)

   service = AS_SERV_SPECIAL(run, args[0])
   iret, output = service.start()
   run.Sortie(iret)

