# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""Definition of MPI_INFO class.
"""

__revision__ = "$Id: as_mpi.py 3322 2008-06-13 16:33:40Z courtois $"


# ----- differ messages translation
def _(mesg):
   return mesg

import os

#-------------------------------------------------------------------------------
class MPI_INFO:
   """Store informations for MPI executions.
   """
   def __init__(self, defines=None):
      """Initialization needs "DEFS" field of ASTER_CONFIG object.
      """
      if defines is not None:
         self._use_mpi = '_USE_MPI' in defines
      else:
         self._use_mpi = False
      self._nbnode = 1
      self._nbcpu  = 1
      self._reptrav = ''
   
   def set_cpuinfo(self, nbnode, nbcpu):
      """Set values of number of processors, of nodes.
      """
      try:
         self._nbcpu = int(nbcpu)
      except ValueError:
         self._nbcpu = 1
      
      try:
         self._nbnode = int(nbnode)
      except ValueError:
         self._nbnode = 1
      
      self._nbnode = min(self._nbnode, self._nbcpu)
      if not self._use_mpi:
         self._nbnode = 1
         self._nbcpu  = 1

#-------------------------------------------------------------------------------
   def really(self):
      """Returns True if Code_Aster executions need mpirun.
      """
      return self._use_mpi

#-------------------------------------------------------------------------------
   def set_rep_trav(self, run, reptrav, basename=''):
      """Set temporary directory for Code_Aster executions.
      """
      if reptrav == '':
         # used shared directory if more than one node
         if self.really() and self.nbnode() > 1:
            dirname = run['shared_tmp']
         else:
            dirname = run['rep_trav']
         reptrav = os.path.join(dirname, basename)
      self._reptrav = reptrav

#-------------------------------------------------------------------------------
   def get_exec_command(self, run, cmd_in):
      """Returns command to run Code_Aster.
      """
      if not self.really():
         return cmd_in

      mpi_script = os.path.join(self.reptrav(), 'mpi_script.sh')
      dict_mpi_args = {
         'mpirun_cmd'         : run['mpirun_cmd'],
         'mpi_get_procid_cmd' : run['mpi_get_procid_cmd'],
         'mpi_hostfile'       : run['mpi_hostfile'],
         'mpi_nbnoeud'        : self.nbnode(),
         'mpi_nbcpu'          : self.nbcpu(),
         'cmd_to_run'         : cmd_in,
         'wrkdir'             : os.getcwd(),
         'program'            : mpi_script,
      }
      template = os.path.join(run.astk_serv_root, 'lib', 'mpirun_template')
      if not run.Exists(template):
         run.Mess(_('file not found : %s'), _('<F>_FILE_NOT_FOUND'))
      content = open(template, 'r').read() % dict_mpi_args
      run.DBG(content)
      open(mpi_script, 'w').write(content)
      os.chmod(mpi_script, 0755)
      return run['mpirun_cmd'] % dict_mpi_args

#-------------------------------------------------------------------------------
   def nbcpu(self):
      return self._nbcpu

#-------------------------------------------------------------------------------
   def nbnode(self):
      return self._nbnode

#-------------------------------------------------------------------------------
   def reptrav(self):
      return self._reptrav


